/*+-------------------------------------------------------------------+
  |                                                   iMAXhpAlert.mq4 |
  |                                                                   |
  | "Modified Optimum Elliptic Filter"                                |
  | Source of calculations:                                           |
  | Stocks & Commodities vol 18:7 p20-29                              |
  | Optimal Detrending by John F. Ehlers                              |
  |                                                                   |
  |                                              Crafted by Wylie     |
  |                                              Copyright  2008     |
  |                                              dazzle.html@live.com |
  +-------------------------------------------------------------------+*/

#property copyright "Copyright  2008, Wylie"
#property link      "dazzle.html@live.com"

#property indicator_chart_window
#property indicator_buffers 4

/*+-------------------------------------------------------------------+
  | iMAXhpAlert Parameters                                            |
  +-------------------------------------------------------------------+*/

extern color    Phase1color     = Red;
extern color    Phase2color     = Aqua;
extern bool     SuppressPhLines = false;
extern bool     EnableAlerts    = true;
extern bool     EnableArrows    = true;
extern int      ArrowUPsize     = 1;
extern int      ArrowDNsize     = 1;
extern color    ArrowUPcolor    = Aqua;
extern color    ArrowDNcolor    = Red;

double          iBuf0[],iBuf1[],CrossUp[],CrossDn[],Clearance,Ph2step;
int             MinBars,AST,c;
string          Chart;
static datetime AlertX[2];

/*+-------------------------------------------------------------------+
  | iMAXhpAlert Initialization                                        |
  +-------------------------------------------------------------------+*/

int init()
{
 IndicatorBuffers(4);

 // Index Buffer 0

 SetIndexBuffer(0,iBuf0);
 SetIndexShift(0,0);
 SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,Phase1color);

 // Index Buffer 1

 SetIndexBuffer(1,iBuf1);
 SetIndexShift(1,0);
 SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,1,Phase2color);

 // Index Buffer 2

 SetIndexBuffer(2,CrossUp);
 SetIndexStyle(2,DRAW_ARROW,STYLE_SOLID,ArrowUPsize,ArrowUPcolor);
 SetIndexArrow(2,221);

 // Index Buffer 3

 SetIndexBuffer(3,CrossDn);
 SetIndexStyle(3,DRAW_ARROW,STYLE_SOLID,ArrowDNsize,ArrowDNcolor);
 SetIndexArrow(3,222);
                                      // The "Clearance" variable below sets the distance from the trendline to the arrows...
                                      // It can be adjusted by the user to their own preference...
 switch(Period())
     {case PERIOD_M1:  Chart = "M1";  Clearance = 0.0003; AST =      59;Ph2step = 0.0001;  break;
      case PERIOD_M5:  Chart = "M5";  Clearance = 0.0007; AST =     259;Ph2step = 0.00015; break;
      case PERIOD_M15: Chart = "M15"; Clearance = 0.0011; AST =     899;Ph2step = 0.00030; break;
      case PERIOD_M30: Chart = "M30"; Clearance = 0.0018; AST =    1799;Ph2step = 0.0006;  break;
      case PERIOD_H1:  Chart = "H1";  Clearance = 0.0027; AST =    3599;Ph2step = 0.0012;  break;
      case PERIOD_H4:  Chart = "H4";  Clearance = 0.0038; AST =   14399;Ph2step = 0.0024;  break;
      case PERIOD_D1:  Chart = "D1";  Clearance = 0.0070; AST =   86399;Ph2step = 0.0048;  break;
      case PERIOD_W1:  Chart = "W1";  Clearance = 0.0130; AST =  604799;Ph2step = 0.0084;  break;
      case PERIOD_MN1: Chart = "MN1"; Clearance = 0.0260; AST = 2591999;Ph2step = 0.0168;  break;}

 IndicatorShortName("iMAXhpAlert");
 MinBars = 20;

 return (0);
} // int init()
  
/*+-------------------------------------------------------------------+
  | iMAXhpAlert Main cycle                                            |
  +-------------------------------------------------------------------+*/

int start()
{    
 if(Bars <= MinBars){return (0);}

 if(SuppressPhLines){SetIndexStyle(0,DRAW_NONE);SetIndexStyle(1,DRAW_NONE);}

 int countedBars = IndicatorCounted();

 if(countedBars < 0){return (-1);}
 if(countedBars > 0){countedBars--;}

 int limit = Bars - countedBars - 1;
 double  x = 0.5;

 c = limit;

 while(c >= 0)
     {iBuf0[c] = (0.13785 * (2 * ((High[c]   + Low[c])   * x) - ((High[c+1] + Low[c+1]) * x)))
               + (0.0007  * (2 * ((High[c+1] + Low[c+1]) * x) - ((High[c+2] + Low[c+2]) * x)))
               + (0.13785 * (2 * ((High[c+2] + Low[c+2]) * x) - ((High[c+3] + Low[c+3]) * x)))
               + (1.2103  * iBuf0[c + 1] - 0.4867 * iBuf0[c + 2]); 

      if(Close[c] > iBuf0[c])
         {iBuf1[c] = iBuf0[c] + Ph2step;}

      if(Close[c] < iBuf0[c])
         {iBuf1[c] = iBuf0[c] - Ph2step;}

      double r0 = iBuf0[c],r1 = iBuf0[c+1],
             b0 = iBuf1[c],b1 = iBuf1[c+1];

      if(EnableArrows)
         {if(b0 > r0 && b1 < r1)
             {CrossUp[c] = r0 - Clearance;}else{CrossUp[c] = EMPTY_VALUE;}
          if(b0 < r0 && b1 > r1)
             {CrossDn[c] = r0 + Clearance;}else{CrossDn[c] = EMPTY_VALUE;}}
      c--;
     } // while(c >= 0)

 if(EnableAlerts)
    {if(b0 > r0 && b1 < r1)
        {sendAlert(0,(getDateTime()+" "+Symbol()+" "+Chart+" iMAXhpAlert signals up trend crossing."));}
     if(b0 < r0 && b1 > r1)
        {sendAlert(1,(getDateTime()+" "+Symbol()+" "+Chart+" iMAXhpAlert signals down trend crossing."));}}

 return (0);  
} // int start()

/*+-------------------------------------------------------------------+
  | End iMAXhpAlert Main cycle                                        |
  +-------------------------------------------------------------------+*/

/*+-------------------------------------------------------------------+
  | iMAXhpAlert support functions                                     |
  +-------------------------------------------------------------------+*/

/*+-------------------------------------------------------------------+
  | *** Alert processing function                                     |
  +-------------------------------------------------------------------+*/

void sendAlert(int AlertNum,string AlertText)
{
 if(TimeCurrent() > AlertX[AlertNum] + AST)
    {AlertX[AlertNum] = TimeCurrent();
                                        /* Add a sound file here if you want another form of alert...
                                           The sound file must be located in the MT4 "sounds" file folder 
                                           and must be a .wav format file. Also remove the // characters before
                                           the PlaySound instruction.  The Alert command and semicolon can 
                                           be removed if the user wishes to remove the alert entirely, if favor 
                                           of a sound file... or it can be "commented out" just add // before 
                                           the alert instruction. */

     PlaySound("alert.wav"); 
     Print(AlertText);
//     Alert(AlertText);
    } // if(TimeCurrent() > AlertX[AlertNum] + AST)

 return(0);
} // void sendAlert(int AlertNum,string AlertText)

/*+-------------------------------------------------------------------+
  | *** Date/Time display function                                    |
  +-------------------------------------------------------------------+*/

string getDateTime()
{
 string dsplyDateTime = TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS);

 return(dsplyDateTime);
} // string getDateTime()

/*+-------------------------------------------------------------------+
  | End iMAXhpAlert support functions                                 |
  +-------------------------------------------------------------------+*/


